@@ -6,22 +6,27 @@ Scatter
 rotated and scaled with two or more fingers on a multitouch system.
 
 Scatter has its own matrix transformation: the modelview matrix is changed
-before the child is drawn, and the previous matrix is restored when the
-drawing is finished. That makes possible the rotation / scale / translation of
-the entire children tree, without changing them.
+before the children are drawn and the previous matrix is restored when the
+drawing is finished. That makes it possible to perform rotation, scaling and
+translation over the entire children tree without changing any widget
+properties.
 
-That specific behavior makes the scatter unique, and there are some advantages /
-constraints that you should consider:
+That specific behavior makes the scatter unique, but there are some
+advantages / constraints that you should consider:
 
-#. The children are positionned relative to 0, 0. The scatter position has
-   no impact of the position of children. This applies to the size too.
-#. If you want to resize the scatter, use scale, not size. (read #1.)
-#. The scatter is not a layout. You must take care of size of children
+#. The children are positioned relative to the scatter similar to a
+   RelativeLayout (see :mod:`~kivy.uix.relativelayout`). So when dragging the
+   scatter, the position of the children don't change, only the position of
+   the scatter.
+#. The scatter size has no impact on the size of the children.
+#. If you want to resize the scatter, use scale, not size. (read #2.). Scale
+   transforms both the scatter and its children, but does not change size.
+#. The scatter is not a layout. You must manage the size of the children
    yourself.
 
 For touch events, the scatter converts from the parent matrix to the scatter
 matrix automatically in on_touch_down/move/up events. If you are doing things
-manually, you will need to use :meth:`~kivy.uix.widget.Widget.to_parent`,
+manually, you will need to use :meth:`~kivy.uix.widget.Widget.to_parent` and
 :meth:`~kivy.uix.widget.Widget.to_local`.
 
 Usage
@@ -58,7 +63,7 @@ Allow only translation on x axis::
 Automatic Bring to Front
 ------------------------
 
-If the :data:`Scatter.auto_bring_to_front` property is True, the scatter
+If the :attr:`Scatter.auto_bring_to_front` property is True, the scatter
 widget will be removed and re-added to the parent when it is touched
 (brought to front, above all other widgets in the parent). This is useful
 when you are manipulating several scatter widgets and don't want the active
@@ -68,7 +73,7 @@ Scale Limitation
 ----------------
 
 We are using a 32-bit matrix in double representation. That means we have
-a limit for scaling. You cannot do infinite scale down/up with our
+a limit for scaling. You cannot do infinite scaling down/up with our
 implementation. Generally, you don't hit the minimum scale (because you don't
 see it on the screen), but the maximum scale is 9.99506983235e+19 (2^66).
 
@@ -80,8 +85,8 @@ Behaviors
 ---------
 
 .. versionchanged:: 1.1.0
-    If no control interactions are enabled, then touch handler will never return
-    True.
+    If no control interactions are enabled, then the touch handler will never
+    return True.
 
 '''
 
@@ -89,7 +94,7 @@ __all__ = ('Scatter', 'ScatterPlane')
 
 from math import radians
 from kivy.properties import BooleanProperty, AliasProperty, \
-        NumericProperty, ObjectProperty
+    NumericProperty, ObjectProperty, BoundedNumericProperty
 from kivy.vector import Vector
 from kivy.uix.widget import Widget
 from kivy.graphics.transformation import Matrix
@@ -97,28 +102,38 @@ from kivy.graphics.transformation import Matrix
 
 class Scatter(Widget):
     '''Scatter class. See module documentation for more information.
+
+    :Events:
+        `on_transform_with_touch`:
+            Fired when the scatter has been transformed by user touch
+            or multitouch, such as panning or zooming.
+
+    .. versionchanged:: 1.8.0
+        Event `on_transform_with_touch` added.
     '''
 
+    __events__ = ('on_transform_with_touch',)
+
     auto_bring_to_front = BooleanProperty(True)
     '''If True, the widget will be automatically pushed on the top of parent
     widget list for drawing.
 
-    :data:`auto_bring_to_front` is a :class:`~kivy.properties.BooleanProperty`,
-    default to True.
+    :attr:`auto_bring_to_front` is a :class:`~kivy.properties.BooleanProperty`
+    and defaults to True.
     '''
 
     do_translation_x = BooleanProperty(True)
-    '''Allow translation on X axis
+    '''Allow translation on the X axis.
 
-    :data:`do_translation_x` is a :class:`~kivy.properties.BooleanProperty`,
-    default to True.
+    :attr:`do_translation_x` is a :class:`~kivy.properties.BooleanProperty` and
+    defaults to True.
     '''
 
     do_translation_y = BooleanProperty(True)
     '''Allow translation on Y axis.
 
-    :data:`do_translation_y` is a :class:`~kivy.properties.BooleanProperty`,
-    default to True.
+    :attr:`do_translation_y` is a :class:`~kivy.properties.BooleanProperty` and
+    defaults to True.
     '''
 
     def _get_do_translation(self):
@@ -129,33 +144,44 @@ class Scatter(Widget):
             self.do_translation_x, self.do_translation_y = value
         else:
             self.do_translation_x = self.do_translation_y = bool(value)
-    do_translation = AliasProperty(_get_do_translation, _set_do_translation,
-                                bind=('do_translation_x', 'do_translation_y'))
-    '''Allow translation on X or Y axis.
+    do_translation = AliasProperty(
+        _get_do_translation, _set_do_translation,
+        bind=('do_translation_x', 'do_translation_y'))
+    '''Allow translation on the X or Y axis.
 
-    :data:`do_translation` is a :class:`~kivy.properties.AliasProperty` of
-    (:data:`do_translation_x` + :data:`do_translation_y`)
+    :attr:`do_translation` is an :class:`~kivy.properties.AliasProperty` of
+    (:attr:`do_translation_x` + :attr:`do_translation_y`)
+    '''
+
+    translation_touches = BoundedNumericProperty(1, min=1)
+    '''Determine whether translation is triggered by a single or multiple
+    touches. This only has effect when :attr:`do_translation` = True.
+
+    :attr:`translation_touches` is a :class:`~kivy.properties.NumericProperty`
+    and defaults to 1.
+
+    .. versionadded:: 1.7.0
     '''
 
     do_rotation = BooleanProperty(True)
     '''Allow rotation.
 
-    :data:`do_rotation` is a :class:`~kivy.properties.BooleanProperty`,
-    default to True.
+    :attr:`do_rotation` is a :class:`~kivy.properties.BooleanProperty` and
+    defaults to True.
     '''
 
     do_scale = BooleanProperty(True)
     '''Allow scaling.
 
-    :data:`do_scale` is a :class:`~kivy.properties.BooleanProperty`,
-    default to True.
+    :attr:`do_scale` is a :class:`~kivy.properties.BooleanProperty` and
+    defaults to True.
     '''
 
     do_collide_after_children = BooleanProperty(False)
     '''If True, the collision detection for limiting the touch inside the
     scatter will be done after dispaching the touch to the children.
-    You can put children outside the bounding box of the scatter, and be able to
-    touch them.
+    You can put children outside the bounding box of the scatter and still be
+    able to touch them.
 
     .. versionadded:: 1.3.0
     '''
@@ -163,29 +189,29 @@ class Scatter(Widget):
     scale_min = NumericProperty(0.01)
     '''Minimum scaling factor allowed.
 
-    :data:`scale_min` is a :class:`~kivy.properties.NumericProperty`, default to
-    0.01
+    :attr:`scale_min` is a :class:`~kivy.properties.NumericProperty` and
+    defaults to 0.01.
     '''
 
     scale_max = NumericProperty(1e20)
     '''Maximum scaling factor allowed.
 
-    :data:`scale_max` is a :class:`~kivy.properties.NumericProperty`, default to
-    1e20
+    :attr:`scale_max` is a :class:`~kivy.properties.NumericProperty` and
+    defaults to 1e20.
     '''
 
     transform = ObjectProperty(Matrix())
     '''Transformation matrix.
 
-    :data:`transform` is a :class:`~kivy.properties.ObjectProperty`, default to
-    the identity matrix.
+    :attr:`transform` is an :class:`~kivy.properties.ObjectProperty` and
+    defaults to the identity matrix.
     '''
 
     transform_inv = ObjectProperty(Matrix())
     '''Inverse of the transformation matrix.
 
-    :data:`transform_inv` is a :class:`~kivy.properties.ObjectProperty`, default
-    to the identity matrix.
+    :attr:`transform_inv` is an :class:`~kivy.properties.ObjectProperty` and
+    defaults to the identity matrix.
     '''
 
     def _get_bbox(self):
@@ -208,7 +234,7 @@ class Scatter(Widget):
         ((x, y), (w, h))
         # x, y = lower left corner
 
-    :data:`bbox` is a :class:`~kivy.properties.AliasProperty`.
+    :attr:`bbox` is an :class:`~kivy.properties.AliasProperty`.
     '''
 
     def _get_rotation(self):
@@ -221,28 +247,40 @@ class Scatter(Widget):
         angle_change = self.rotation - rotation
         r = Matrix().rotate(-radians(angle_change), 0, 0, 1)
         self.apply_transform(r, post_multiply=True,
-                            anchor=self.to_local(*self.center))
+                             anchor=self.to_local(*self.center))
     rotation = AliasProperty(_get_rotation, _set_rotation, bind=(
         'x', 'y', 'transform'))
     '''Rotation value of the scatter.
 
-    :data:`rotation` is a :class:`~kivy.properties.AliasProperty`.
+    :attr:`rotation` is an :class:`~kivy.properties.AliasProperty`.
     '''
 
     def _get_scale(self):
         p1 = Vector(*self.to_parent(0, 0))
         p2 = Vector(*self.to_parent(1, 0))
         scale = p1.distance(p2)
-        return float(scale)
+
+        # XXX float calculation are not accurate, and then, scale can be
+        # throwed again even with only the position change. So to
+        # prevent anything wrong with scale, just avoid to dispatch it
+        # if the scale "visually" didn't change. #947
+        # Remove this ugly hack when we'll be Python 3 only.
+        if hasattr(self, '_scale_p'):
+            if str(scale) == str(self._scale_p):
+                return self._scale_p
+
+        self._scale_p = scale
+        return scale
 
     def _set_scale(self, scale):
         rescale = scale * 1.0 / self.scale
         self.apply_transform(Matrix().scale(rescale, rescale, rescale),
-                        post_multiply=True, anchor=self.to_local(*self.center))
+                             post_multiply=True,
+                             anchor=self.to_local(*self.center))
     scale = AliasProperty(_get_scale, _set_scale, bind=('x', 'y', 'transform'))
     '''Scale value of the scatter.
 
-    :data:`scale` is a :class:`~kivy.properties.AliasProperty`.
+    :attr:`scale` is an :class:`~kivy.properties.AliasProperty`.
     '''
 
     def _get_center(self):
@@ -341,18 +379,18 @@ class Scatter(Widget):
 
     def apply_transform(self, trans, post_multiply=False, anchor=(0, 0)):
         '''
-        Transforms scatter by trans (on top of its current transformation
+        Transforms the scatter by trans (on top of its current transformation
         state).
 
         :Parameters:
             `trans`: transformation matrix from transformation lib.
-                Transformation to be applied to the scatter widget
-            `anchor`: tuple, default to (0, 0)
+                Transformation to be applied to the scatter widget.
+            `anchor`: tuple, defaults to (0, 0).
                 The point to use as the origin of the transformation
-                (uses local widget space)
-            `post_multiply`: bool, default to False
-                If true the transform matrix is post multiplied
-                (as if applied before the current transform)
+                (uses local widget space).
+            `post_multiply`: bool, defaults to False.
+                If True, the transform matrix is post multiplied
+                (as if applied before the current transform).
         '''
         t = Matrix().translate(anchor[0], anchor[1], 0)
         t = t.multiply(trans)
@@ -365,34 +403,45 @@ class Scatter(Widget):
 
     def transform_with_touch(self, touch):
         # just do a simple one finger drag
-        if len(self._touches) == 1:
+        changed = False
+        if len(self._touches) == self.translation_touches:
             # _last_touch_pos has last pos in correct parent space,
             # just like incoming touch
             dx = (touch.x - self._last_touch_pos[touch][0]) \
-                    * self.do_translation_x
+                * self.do_translation_x
             dy = (touch.y - self._last_touch_pos[touch][1]) \
-                    * self.do_translation_y
+                * self.do_translation_y
+            dx = dx / self.translation_touches
+            dy = dy / self.translation_touches
             self.apply_transform(Matrix().translate(dx, dy, 0))
-            return
+            changed = True
 
-        # we have more than one touch...
-        points = [Vector(self._last_touch_pos[t]) for t in self._touches]
+        if len(self._touches) == 1:
+            return changed
+
+        # we have more than one touch... list of last known pos
+        points = [Vector(self._last_touch_pos[t]) for t in self._touches
+                  if t is not touch]
+        # add current touch last
+        points.append(Vector(touch.pos))
 
         # we only want to transform if the touch is part of the two touches
-        # furthest apart! So first we find anchor, the point to transform
-        # around as the touch farthest away from touch
-        anchor = max(points, key=lambda p: p.distance(touch.pos))
+        # farthest apart! So first we find anchor, the point to transform
+        # around as another touch farthest away from current touch's pos
+        anchor = max(points[:-1], key=lambda p: p.distance(touch.pos))
 
         # now we find the touch farthest away from anchor, if its not the
         # same as touch. Touch is not one of the two touches used to transform
         farthest = max(points, key=anchor.distance)
-        if points.index(farthest) != self._touches.index(touch):
-            return
+        if farthest is not points[-1]:
+            return changed
 
         # ok, so we have touch, and anchor, so we can actually compute the
         # transformation
         old_line = Vector(*touch.ppos) - anchor
         new_line = Vector(*touch.pos) - anchor
+        if not old_line.length():   # div by zero
+            return changed
 
         angle = radians(new_line.angle(old_line)) * self.do_rotation
         self.apply_transform(Matrix().rotate(angle, 0, 0, 1), anchor=anchor)
@@ -400,10 +449,14 @@ class Scatter(Widget):
         if self.do_scale:
             scale = new_line.length() / old_line.length()
             new_scale = scale * self.scale
-            if new_scale < self.scale_min or new_scale > self.scale_max:
-                scale = 1.0
+            if new_scale < self.scale_min:
+                scale = self.scale_min / self.scale
+            elif new_scale > self.scale_max:
+                scale = self.scale_max / self.scale
             self.apply_transform(Matrix().scale(scale, scale, scale),
                                  anchor=anchor)
+            changed = True
+        return changed
 
     def _bring_to_front(self):
         # auto bring to front
@@ -432,9 +485,9 @@ class Scatter(Widget):
         # if our child didn't do anything, and if we don't have any active
         # interaction control, then don't accept the touch.
         if not self.do_translation_x and \
-            not self.do_translation_y and \
-            not self.do_rotation and \
-            not self.do_scale:
+                not self.do_translation_y and \
+                not self.do_rotation and \
+                not self.do_scale:
             return False
 
         if self.do_collide_after_children:
@@ -462,13 +515,28 @@ class Scatter(Widget):
 
         # rotate/scale/translate
         if touch in self._touches and touch.grab_current == self:
-            self.transform_with_touch(touch)
+            if self.transform_with_touch(touch):
+                self.dispatch('on_transform_with_touch', touch)
             self._last_touch_pos[touch] = touch.pos
 
         # stop propagating if its within our bounds
         if self.collide_point(x, y):
             return True
 
+    def on_transform_with_touch(self, touch):
+        '''
+        Called when a touch event has transformed the scatter widget.
+        By default this does nothing, but can be overriden by derived
+        classes that need to react to transformations caused by user
+        input.
+
+        :Parameters:
+            `touch`: the touch object which triggered the transformation.
+
+        .. versionadded:: 1.8.0
+        '''
+        pass
+
     def on_touch_up(self, touch):
         x, y = touch.x, touch.y
         # if the touch isnt on the widget we do nothing, just try children
@@ -492,8 +560,8 @@ class Scatter(Widget):
 
 
 class ScatterPlane(Scatter):
-    '''This is essentially an unbounded Scatter widget, convenience
-       class to make easier the writting of infinite planes
+    '''This is essentially an unbounded Scatter widget: it's a convenience
+       class to make it easier to handle infinite planes.
     '''
 
     def __init__(self, **kwargs):
